
using Jypeli;

namespace pomppu_possu;

// TODO: Taulukko, silmukka ja funktio, Leo, ks: https://tim.jyu.fi/answers/kurssit/tie/ohj1/v/2025/kevat/demot/demo6?answerNumber=1&task=TeeTaulukko&user=leosaksa
// TODO: Taulukko, silmukka ja funktio, Suvi, ks: https://tim.jyu.fi/answers/kurssit/tie/ohj1/v/2025/kevat/demot/demo6?answerNumber=2&task=TeeTaulukko&user=suvleean

/// @author suvitoivanen, leosaksa
/// @version 02.04.2025
/// <summary>
/// Eeppinen tasohyppely peli, jossa pelaajan tehtävä on kerätä kaikki
/// kartalta löytyvät megaporkkanat
/// </summary>
public class PomppuPossu : PhysicsGame
{
    private const double Nopeus = 200;
    private const double Hyppynopeus = 750;
    private const int RuudunKoko = 60;

    private PlatformCharacter _pomppuPossu;
    private Image _pomppuPossunKuva = LoadImage("pomppupossu.png");
    private Image _porkkanaKuva = LoadImage("porkkananKuva3.png");

    private SoundEffect _pisteAani = LoadSoundEffect("maali.wav");

    private IntMeter _pelaajanPisteet;
    
    
/// <summary>
/// Pääohjelma kutsuu kentän luovaa aliohjelmaa ja luo näppäinkomennot.
/// </summary>
    public override void Begin()
    {
        
        LuoKentta();
        LisaaLaskuri();
        
        Camera.Follow(_pomppuPossu);
        Camera.ZoomFactor = 1.2;
        Camera.StayInLevel = true;
        MasterVolume = 0.5;
        Level.CreateBorders();
        

        Keyboard.Listen(Key.Up, ButtonState.Pressed, Hyppaa, "Pelaaja hyppää", _pomppuPossu, Hyppynopeus);
        Keyboard.Listen(Key.Left, ButtonState.Down, Liikuta, "Liikuttaa pelaajaa vasemmalle", _pomppuPossu, -Nopeus);
        Keyboard.Listen(Key.Right, ButtonState.Down, Liikuta, "Liikuttaa pelaajaa oikealle", _pomppuPossu, Nopeus);
        
        Keyboard.Listen(Key.Escape, ButtonState.Pressed, ConfirmExit, "Lopeta peli");
    }


    /// <summary>
    /// Funktio luo kentän hyödyntäen txt tiedostoa ja lisää painovoiman.
    /// </summary>
    private void LuoKentta()
    {
        TileMap kentta = TileMap.FromLevelAsset("kentta1.txt");
        Gravity = new Vector(0, -1000);
        kentta.SetTileMethod('#', LisaaTaso);
        kentta.SetTileMethod('N', LisaaPelaaja);
        kentta.SetTileMethod('*', LisaaPorkkana);
        kentta.Execute(RuudunKoko, RuudunKoko);
        Level.Background.CreateGradient(Color.White, Color.SkyBlue);
    }
    
    
   /// <summary>
   /// Funktio lisää kerättävät objektit kentälle
   /// </summary>
   /// <param name="paikka">Mihin objekti tulee kentällä</param>
   /// <param name="leveys">Leveys</param>
   /// <param name="korkeus">Korkeus</param>
    private void LisaaPorkkana(Vector paikka, double leveys, double korkeus)
    {
        Megaporkkanat megaporkkana = new Megaporkkanat(60, 60);
        megaporkkana.Position = paikka;
        megaporkkana.MakeStatic();
        megaporkkana.Image = _porkkanaKuva;
        Add(megaporkkana);
    }
   
   
/// <summary>
/// Funktio lisää tasot kentälle.
/// </summary>
/// <param name="paikka">paikka</param>
/// <param name="leveys">leveys</param>
/// <param name="korkeus">korkeus</param>
    private void LisaaTaso(Vector paikka, double leveys, double korkeus)
    {
        Tasot taso = new Tasot(leveys, 0.5*korkeus);
        taso.Position = paikka;
        taso.Color = Color.SpringGreen;
        taso.MakeStatic();
        Add(taso);
    }


/// <summary>
/// Funktio lisää pelaajan kentälle.
/// </summary>
/// <param name="paikka">paikka</param>
/// <param name="leveys">leveys</param>
/// <param name="korkeus">korkeus</param>
    private void LisaaPelaaja(Vector paikka, double leveys, double korkeus)
    {
        _pomppuPossu = new PlatformCharacter(80, 100);
        _pomppuPossu.Position = paikka;
        _pomppuPossu.Mass = 4.0;
        _pomppuPossu.Image = _pomppuPossunKuva;
        AddCollisionHandler<PhysicsObject, Megaporkkanat>(_pomppuPossu, TormattiinEsineeseen);
        Add(_pomppuPossu);
    }


/// <summary>
/// Funktio luo liikkeen sivuille pelaajalle, jota kutsutaan näppäimistöllä.
/// </summary>
/// <param name="pomppuPossu">pelaaja</param>
/// <param name="nopeus">liike</param>
    private void Liikuta(PlatformCharacter pomppuPossu, double nopeus)
    {
        pomppuPossu.Walk(nopeus);
    }


/// <summary>
/// Funktio luo liikkeen ylöspäin, jota kutsutaan näppäimistöllä.
/// </summary>
/// <param name="pomppuPossu">pelaaja</param>
/// <param name="nopeus">liike</param>
    private void Hyppaa(PlatformCharacter pomppuPossu, double nopeus)
    {
        pomppuPossu.Jump(nopeus);
    }


/// <summary>
/// Funktio käsittelee törmäykset.
/// Törmäyksen yhteydessä tuhotaan porkkana,
/// soitetaan äänimerkki, näytetään viesti onnistuneesta
/// keräyksestä, sekä lisätään yksi piste pistelaskuriin
/// </summary>
/// <param name="tormaaja">pelaaja</param>
/// <param name="kohde">kerättävät pisteet</param>
    public void TormattiinEsineeseen(PhysicsObject tormaaja, Megaporkkanat kohde)
    {
        kohde.OtaVastaanOsuma();
        _pisteAani.Play();
        MessageDisplay.Add("Megaporkkana kerätty!");
        _pelaajanPisteet.Value += 1;
      
    }


    /// <summary>
    /// Liikuttaa fysiikkaoliota.
    /// </summary>
    /// <param name="liikuteltava">Liikutettava</param>
    /// <param name="suunta">Suunta</param>
    public void Liikuta(PhysicsObject liikuteltava, Vector suunta)
    {
        liikuteltava.Push(suunta);
    }
    
    
    /// <summary>
    /// Aliohjelma luo pistelaskurin
    /// </summary>
    /// <param name="x">laskurin x-koordinaatti näytöllä</param>
    /// <param name="y">laskurin y-koordinaatti näytöllä</param>
    /// <returns>Laskuri</returns>
    IntMeter LuoPisteLaskuri(double x, double y)
    {
        IntMeter laskuri = new IntMeter(0);
       

        Label naytto = new Label();
        naytto.BindTo(laskuri);
        naytto.X = x;
        naytto.Y = y;
        naytto.TextColor = Color.Black;
        naytto.BorderColor = Level.Background.Color;
        naytto.Color = Level.Background.Color;
        Add(naytto);
        
        return laskuri;
    }
    
    
/// <summary>
/// Lisää Laskurin peliin ja määrittää sen sijainnin näytöllä
/// </summary>
    void LisaaLaskuri()
    {
        _pelaajanPisteet = LuoPisteLaskuri(Screen.Right - 100.0, Screen.Top - 100.0);
    }

    
}


